#include "../stdafx.h"
#include <string.h>
#include "MultiDllInput.h"
#include "../PlaylistManager.h"
#include "../MainWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CMainWnd* pMainFrame;
extern PlaylistManager m_PlaylistManager;

CMultiDllInput::~CMultiDllInput()
{
	do
	{
		currentDllInput = (CDllInput *)listDllInput[listDllInput.GetSize() - 1];
		currentDllInput->Quit();
		delete currentDllInput;
		currentDllInput = NULL;
		listDllInput.SetSize(listDllInput.GetSize() - 1);
	}
	while (listDllInput.GetSize() != 0);
}

CMultiDllInput::CMultiDllInput(HWND hWnd)
{
#ifndef _UNICODE
	CString dllInName = "";
	CString dllInKey = "";
	bool alreadyLoad = false;
	int i = 0;
	do
	{
		listDllInput.SetSize(listDllInput.GetSize() + 1);
		dllInKey.Format(_T("dllInName%d"), i);
		dllInName = AfxGetApp()->GetProfileString(_T("Preferences"), dllInKey, _T(""));
		if (dllInName == "NONE")
			dllInName = "";
		if (((dllInName == "" || dllInName == "STANDARD") && !alreadyLoad) || ((dllInName != "" && dllInName != "STANDARD")))
		{
			if (dllInName == "STANDARD" || dllInName == "")
			{
				currentDllInput = new CDllInput("", hWnd);
				alreadyLoad = true;
			}
			else
				currentDllInput = new CDllInput(dllInName, hWnd);
			if (dllInName != currentDllInput->GetFilePath() && dllInName != "STANDARD" && dllInName != "")
			{
				delete currentDllInput;
				listDllInput.SetSize(listDllInput.GetSize() - 1);
				currentDllInput = NULL;
			}
			else
			{
				currentDllInput->Init();
				listDllInput[listDllInput.GetSize() - 1] = (CObject *)currentDllInput;
			}
		}
		else
			listDllInput.SetSize(listDllInput.GetSize() - 1);
		i++;
	}
	while (dllInName.GetLength() != 0);
#else
	listDllInput.SetSize(listDllInput.GetSize() + 1);
	currentDllInput = new CDllInput("", hWnd);
	currentDllInput->Init();
	listDllInput[listDllInput.GetSize() - 1] = (CObject *)currentDllInput;
#endif
	currentDllInput = (CDllInput *)listDllInput[0];
}

void CMultiDllInput::GetFileInfo(TCHAR* file, TCHAR* title, int* length_in_ms) // if file == NULL, current playing is used
{
	try
	{
		CDllInput* dllInfo;
		int res = -1;
		TCHAR fn[5000];
		_tcscpy(fn, file);
		TCHAR* cFn = _tcsrev(_tcslwr(_tcsdup(fn)));

		for (int i = 0; i < listDllInput.GetSize(); i++)
		{
			dllInfo = (CDllInput *)listDllInput[i];
			if (dllInfo->IsOurFile(fn) != 0)
			{
				dllInfo->GetFileInfo(fn, title, length_in_ms);
				res = 0;
			}
			if (res != -1)
				break;
			TCHAR* pExtToken = _tcsrev(_tcslwr(_tcsdup(dllInfo->GetFileExtensions())));
			TCHAR* extToken = _tcstok(pExtToken, _T(";"));
			while (extToken != NULL)
			{
				if (_tcsstr(cFn, extToken) == cFn)
				{
					dllInfo->GetFileInfo(fn, title, length_in_ms);
					res = 0;
					break;
				}
				extToken = _tcstok(NULL, _T(";"));
			}
			delete pExtToken;
			if (res != -1)
				break;
		}
		delete cFn;
	}
	catch (...)
	{
		*length_in_ms = 0;
		title[0] = 0;
	}
}

int CMultiDllInput::InfoBox(TCHAR* file, HWND hwndParent)
{
	int res = -1;
	TCHAR fn[5000];
	_tcscpy(fn, file);
	TCHAR* cFn = _tcsrev(_tcslwr(_tcsdup(fn)));

	for (int i = 0; i < listDllInput.GetSize(); i++)
	{
		currentDllInput = (CDllInput *)listDllInput[i];
		if (currentDllInput->IsOurFile(fn) != 0)
		{
			TRACE(_T("%s"),currentDllInput->GetDescription());
			res = currentDllInput->InfoBox(file, hwndParent);
			break;
		}
		TCHAR* pExtToken = _tcsrev(_tcslwr(_tcsdup(currentDllInput->GetFileExtensions())));
		TCHAR* extToken = _tcstok(pExtToken, _T(";"));
		while (extToken != NULL)
		{
			if (_tcsstr(cFn, extToken) == cFn)
			{
				TRACE(_T("%s"),currentDllInput->GetDescription());
				res = currentDllInput->InfoBox(file, hwndParent);
				break;
			}
			extToken = _tcstok(NULL, _T(";"));
		}
		delete pExtToken;
		if (res != -1)
			break;
	}
	delete cFn;
	return res;
}

int CMultiDllInput::IsOurFile(TCHAR* fn) // called before extension checks, to allow detection of mms://, etc
{
	int res = 0;
	for (int i = 0; i < listDllInput.GetSize(); i++)
	{
		CDllInput* curDllInput = (CDllInput*)listDllInput[i];
		res = curDllInput->IsOurFile(fn);
		if (res)
			break;
	}
	return res;
}

// playback stuff
int CMultiDllInput::Play(TCHAR* fn1)	// return zero on success, -1 on file-not-found, some other value on other (stopping winamp) error
{
	int res = -1;
	TCHAR fn[5000];
	_tcscpy(fn, fn1);
	TCHAR* cFn = _tcsrev(_tcslwr(_tcsdup(fn)));

	if (currentDllInput->GetPlayerMode() != PLAYER_MODE_CLOSE)
		currentDllInput->Stop();
	for (int i = 0; i < listDllInput.GetSize(); i++)
	{
		currentDllInput = (CDllInput *)listDllInput[i];
		if (currentDllInput->IsOurFile(fn) != 0)
		{
			TRACE(_T("%s"),currentDllInput->GetDescription());
			res = currentDllInput->Play(fn) + 2;
		}
		if (res != -1)
			break;
		TCHAR* pExtToken = _tcsrev(_tcslwr(_tcsdup(currentDllInput->GetFileExtensions())));
		TCHAR* extToken = _tcstok(pExtToken, _T(";"));
		while (extToken != NULL)
		{
			if (_tcsstr(cFn, extToken) == cFn)
			{
				TRACE(_T("%s"),currentDllInput->GetDescription());
				res = currentDllInput->Play(fn);
				//  			  TRACE(_T("res %d"), res);
				CachedTag cTag = m_PlaylistManager.Get(pMainFrame->Current);
				cTag.SetDuration(currentDllInput->GetLength());
				m_PlaylistManager.Add(cTag, pMainFrame->Current);
				break;
			}
			extToken = _tcstok(NULL, _T(";"));
		}
		delete pExtToken;
		if (res != -1)
			break;
	}
	if (res > 1)
		res -= 2;
	delete cFn;
	pMainFrame->DisplayTooltip();
	if(res!=0)
		TRACE(_T("Lecture impossible"));
	return res;
}

int CMultiDllInput::winampGetExtendedFileInfo(extendedFileInfoStruct ExtendedFileInfo)
{
	int res = -1;
	TCHAR fn[5000];
	_tcscpy(fn, ExtendedFileInfo.filename);
	TCHAR* cFn = _tcsrev(_tcslwr(_tcsdup(fn)));

	for (int i = 0; i < listDllInput.GetSize(); i++)
	{
		currentDllInput = (CDllInput *)listDllInput[i];
		if (currentDllInput->IsOurFile(fn) != 0)
		{
			TRACE(_T("%s"),currentDllInput->GetDescription());
			res = currentDllInput->winampGetExtendedFileInfo(ExtendedFileInfo);
			break;
		}
		TCHAR* pExtToken = _tcsrev(_tcslwr(_tcsdup(currentDllInput->GetFileExtensions())));
		TCHAR* extToken = _tcstok(pExtToken, _T(";"));
		while (extToken != NULL)
		{
			if (_tcsstr(cFn, extToken) == cFn)
			{
				TRACE(_T("%s"),currentDllInput->GetDescription());
				res = currentDllInput->winampGetExtendedFileInfo(ExtendedFileInfo);
				break;
			}
			extToken = _tcstok(NULL, _T(";"));
		}
		delete pExtToken;
		if (res != -1)
			break;
	}
	delete cFn;
	return res;
}
void CMultiDllInput::Pause()	 // pause stream
{
	currentDllInput->Pause();
}

void CMultiDllInput::UnPause()     // unpause stream
{
	currentDllInput->UnPause();
}

int CMultiDllInput::IsPaused()  	// ispaused? return 1 if paused, 0 if not
{
	return currentDllInput->IsPaused();
}

void CMultiDllInput::Stop() 	   // stop (unload) stream
{
	currentDllInput->Stop();
}
// time stuff
int CMultiDllInput::GetLength() 	// get length in ms
{
	int res = 0;
	try
	{
		res = currentDllInput->GetLength();
	}
	catch (...)
	{
	}
	return res;
}

int CMultiDllInput::GetOutputTime()   // returns current output time in ms. (usually returns outMod->GetOutputTime()
{
	int res = 0;
	try
	{
		res = currentDllInput->GetOutputTime();
	}
	catch (...)
	{
	}
	return res;
}

void CMultiDllInput::SetOutputTime(int time_in_ms) // seeks to point in stream (in ms). Usually you signal yoru thread to seek, which seeks and calls outMod->Flush()..
{
	try
	{
		currentDllInput->SetOutputTime(time_in_ms);
	}
	catch (...)
	{
	}
}

// volume stuff
void CMultiDllInput::SetVolume(int volume) // from 0 to 255.. usually just call outMod->SetVolume
{
	try
	{
		currentDllInput->SetVolume(volume);
	}
	catch (...)
	{
	}
}

void CMultiDllInput::SetPan(int pan) // from -127 to 127.. usually just call outMod->SetPan
{
	try
	{
		currentDllInput->SetPan(pan);
	}
	catch (...)
	{
	}
}

// info setting (filled in by winamp)
void CMultiDllInput::SetInfo(int bitrate, int srate, int stereo, int synched) // if -1, changes ignored? :)
{
	try
	{
		currentDllInput->SetInfo(bitrate, srate, stereo, synched);
	}
	catch (...)
	{
	}
}

int CMultiDllInput::GetVersion()		// module type (IN_VER)
{
	int res = 0;
	try
	{
		res = currentDllInput->GetIn_Module()->version;
	}
	catch (...)
	{
	}
	return res;
}
TCHAR* CMultiDllInput::GetDescription() 	// description of module, with version string
{
	try
	{
		return currentDllInput->GetIn_Module()->description;
	}
	catch (...)
	{
		return _T("");
	}
}

void CMultiDllInput::SethMainWindow(HWND hWnd)   // winamp's main window (filled in by winamp)
{
	for (int i = 0; i < listDllInput.GetSize(); i++)
	{
		CDllInput* curDllInput = (CDllInput*)listDllInput[i];
		curDllInput->SethMainWindow(hWnd);
	}
}

CString CMultiDllInput::GetFileExtensions()   // "mp3\0Layer 3 MPEG\0mp2\0Layer 2 MPEG\0mpg\0Layer 1 MPEG\0"
{
	CString res = "",ext;
	for (int i = 0; i < listDllInput.GetSize(); i++)
	{
		CDllInput* curDllInput = (CDllInput*)listDllInput[i];
		ext = curDllInput->GetFileExtensions();
		if (ext != "")
			res = res + ext + _T(";");
	}
	return res;
}

int CMultiDllInput::isSeekable()	  // is this stream seekable?
{
	int res = 0;
	try
	{
		res = currentDllInput->GetIn_Module()->is_seekable;
	}
	catch (...)
	{
	}
	return res;
}

int CMultiDllInput::isUsesOutputPlug()  	// does this plug-in use the output plug-ins? (musn't ever change, ever :)
{
	int res = 0;
	try
	{
		res = currentDllInput->GetIn_Module()->UsesOutputPlug;
	}
	catch (...)
	{
	}
	return res;
}

void CMultiDllInput::SetOutMod(Out_Module* out) // filled in by winamp, optionally used :)
{
	for (int i = 0; i < listDllInput.GetSize(); i++)
	{
		CDllInput* curDllInput = (CDllInput*)listDllInput[i];
		curDllInput->SetOutMod(out);
	}
}

In_Module* CMultiDllInput::GetIn_Module()
{
	return currentDllInput->GetIn_Module();
}

int CMultiDllInput::GetPlayerMode()
{
	int ret = PLAYER_MODE_CLOSE;
	if (currentDllInput != NULL)
		ret = currentDllInput->GetPlayerMode();
	return ret;
}

void CMultiDllInput::SetPlayerMode(int statut)
{
	currentDllInput->SetPlayerMode(statut);
}

CString CMultiDllInput::GetFileExtensionsFilter()
{
	TCHAR* extensions;
	CString ext;
	CString extDesc;
	int pos;
	int startExt;
	int startExtDesc;
	CString allSupported = "All supported files|";
	CString res;

	for (int i = 0; i < listDllInput.GetSize(); i++)
	{
		CDllInput* curDllInput = (CDllInput*)listDllInput[i];
		extensions = curDllInput->GetFileExtensions();
		ext = extensions;
		pos = 0;
		startExt = 0;
		startExtDesc = ext.GetLength() + 1;
		do
		{
			if (*(extensions + pos) == ';')
			{
				ext = extensions;
				ext = ext.Mid(startExt, pos - startExt);
				startExt = pos + 1;
				extDesc = extensions + startExtDesc;
				startExtDesc += extDesc.GetLength() + 1;
				allSupported += "*." + ext + ";";
				if (extDesc == "")
					res += ";*." + ext;
				else
					res += extDesc + "|*." + ext;
			}
			pos++;
		}
		while (*(extensions + pos) != '\0');
		ext = extensions;
		ext = ext.Mid(startExt, pos - startExt);
		if (ext.GetLength() != 0)
		{
			extDesc = extensions + startExtDesc;
			allSupported += "*." + ext + ";";
			if (extDesc == "")
				res += ";*." + ext + "|";
			else
				res += extDesc + "|*." + ext + "|";
		}
	}

	res = allSupported + "|" + res;
	return res;
}

int CMultiDllInput::GetSize() // return the number of loaded input
{
	return listDllInput.GetSize();
}

CDllInput* CMultiDllInput::getDllInput(int pos) //Return the DllInput at pos
{
	return (CDllInput *)listDllInput[pos];
}

void CMultiDllInput::UnLoad(int pos) //Unload the DllInput at pos
{
#ifndef _UNICODE
	if (pos < listDllInput.GetSize())
	{
		CDllInput* input = (CDllInput*)listDllInput[pos];
		input->Quit();
		delete input;
		for (int i = pos; i < listDllInput.GetSize() - 1; i++)
			listDllInput[i] = listDllInput[i + 1];
		listDllInput.SetSize(listDllInput.GetSize() - 1);
		currentDllInput = (CDllInput *)listDllInput[0];
	}
#endif
}

void CMultiDllInput::Load(CString file, HWND hWnd) //Load the DllInput
{
#ifndef _UNICODE
	CDllInput* input = new CDllInput(file, hWnd);
	if (input->GetFilePath() != "STANDARD")
	{
		input->Init();
		listDllInput.SetSize(listDllInput.GetSize() + 1);
		listDllInput[listDllInput.GetSize() - 1] = input;
	}
	else
		delete input;
#endif
}

void CMultiDllInput::UpDllInput(int pos) //Up the DllInput in the list
{
#ifndef _UNICODE
	if (pos == 0)
		return;
	CDllInput* input = (CDllInput*)listDllInput[pos];
	listDllInput[pos] = listDllInput[pos - 1];
	listDllInput[pos - 1] = input;
#endif
}
